@@ -1,6 +1,6 @@
 /********************************************
 bi_funct.c
-copyright 2008-2009,2010, Thomas E. Dickey
+copyright 2008-2010,2012, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.c,v 1.51 2010/12/10 17:00:00 tom Exp $
+ * $MawkId: bi_funct.c,v 1.61 2012/11/29 09:32:21 tom Exp $
  * @Log: bi_funct.c,v @
  * Revision 1.9  1996/01/14  17:16:11  mike
  * flush_all_output() before system()
@@ -63,43 +63,61 @@ the GNU General Public License, version 2, 1991.
  * 1.1 pre-release
  */
 
-#include "mawk.h"
-#include "bi_funct.h"
-#include "bi_vars.h"
-#include "memory.h"
-#include "init.h"
-#include "files.h"
-#include "fin.h"
-#include "field.h"
-#include "regexp.h"
-#include "repl.h"
+#include <mawk.h>
+#include <bi_funct.h>
+#include <bi_vars.h>
+#include <memory.h>
+#include <init.h>
+#include <files.h>
+#include <fin.h>
+#include <field.h>
+#include <regexp.h>
+#include <repl.h>
 
 #include <ctype.h>
 #include <math.h>
+#include <time.h>
+
+#if OPT_TRACE > 0
+#define return_CELL(func, cell) TRACE(("..." func " ->")); \
+				TRACE_CELL(cell); \
+				return cell
+#else
+#define return_CELL(func, cell) return cell
+#endif
 
 /* global for the disassembler */
 /* *INDENT-OFF* */
 BI_REC bi_funct[] =
 {				/* info to load builtins */
 
-   { "length",   bi_length,  0, 1 },	/* special must come first */
-   { "index",    bi_index,   2, 2 },
-   { "substr",   bi_substr,  2, 3 },
-   { "sprintf",  bi_sprintf, 1, 255 },
-   { "sin",      bi_sin,     1, 1 },
-   { "cos",      bi_cos,     1, 1 },
-   { "atan2",    bi_atan2,   2, 2 },
-   { "exp",      bi_exp,     1, 1 },
-   { "log",      bi_log,     1, 1 },
-   { "int",      bi_int,     1, 1 },
-   { "sqrt",     bi_sqrt,    1, 1 },
-   { "rand",     bi_rand,    0, 0 },
-   { "srand",    bi_srand,   0, 1 },
-   { "close",    bi_close,   1, 1 },
-   { "system",   bi_system,  1, 1 },
-   { "toupper",  bi_toupper, 1, 1 },
-   { "tolower",  bi_tolower, 1, 1 },
-   { "fflush",   bi_fflush,  0, 1 },
+   { "length",   bi_length,   0, 1 },	/* special must come first */
+   { "index",    bi_index,    2, 2 },
+   { "substr",   bi_substr,   2, 3 },
+   { "sprintf",  bi_sprintf,  1, 255 },
+   { "sin",      bi_sin,      1, 1 },
+   { "cos",      bi_cos,      1, 1 },
+   { "atan2",    bi_atan2,    2, 2 },
+   { "exp",      bi_exp,      1, 1 },
+   { "log",      bi_log,      1, 1 },
+   { "int",      bi_int,      1, 1 },
+   { "sqrt",     bi_sqrt,     1, 1 },
+   { "rand",     bi_rand,     0, 0 },
+   { "srand",    bi_srand,    0, 1 },
+   { "close",    bi_close,    1, 1 },
+   { "system",   bi_system,   1, 1 },
+   { "toupper",  bi_toupper,  1, 1 },
+   { "tolower",  bi_tolower,  1, 1 },
+   { "fflush",   bi_fflush,   0, 1 },
+
+   /* useful gawk extension (time functions) */
+   { "systime",  bi_systime,  0, 0 },
+#ifdef HAVE_MKTIME
+   { "mktime",   bi_mktime,   1, 1 },
+#endif
+#ifdef HAVE_STRFTIME
+   { "strftime", bi_strftime, 0, 3 },
+#endif
 
    { (char *)    0, (PF_CP) 0, 0, 0 }
 };
@@ -123,13 +141,15 @@ bi_funct_init(void)
 	stp->stval.bip = p;
     }
 
+#ifndef NO_INIT_SRAND
     /* seed rand() off the clock */
     {
 	CELL c;
 
-	c.type = 0;
+	c.type = C_NOINIT;
 	bi_srand(&c);
     }
+#endif
 
 }
 
@@ -142,6 +162,8 @@ bi_length(CELL * sp)
 {
     size_t len;
 
+    TRACE_FUNC("bi_length", sp);
+
     if (sp->type == 0)
 	cellcpy(sp, field);
     else
@@ -155,7 +177,7 @@ bi_length(CELL * sp)
     sp->type = C_DOUBLE;
     sp->dval = (double) len;
 
-    return sp;
+    return_CELL("bi_length", sp);
 }
 
 char *
@@ -209,6 +231,8 @@ bi_index(CELL * sp)
     size_t len;
     const char *p;
 
+    TRACE_FUNC("bi_index", sp);
+
     sp--;
     if (TEST2(sp) != TWO_STRINGS)
 	cast2_to_s(sp);
@@ -228,7 +252,7 @@ bi_index(CELL * sp)
     free_STRING(string(sp + 1));
     sp->type = C_DOUBLE;
     sp->dval = (double) idx;
-    return sp;
+    return_CELL("bi_index", sp);
 }
 
 /*  substr(s, i, n)
@@ -242,6 +266,8 @@ bi_substr(CELL * sp)
     register int i, n;
     STRING *sval;		/* substr(sval->str, i, n) */
 
+    TRACE_FUNC("bi_substr", sp);
+
     n_args = sp->type;
     sp -= n_args;
     if (sp->type != C_STRING)
@@ -255,7 +281,7 @@ bi_substr(CELL * sp)
 	    cell_destroy(sp + 2);
 	}
 	cell_destroy(sp + 1);
-	return sp;
+	return_CELL("bi_substr", sp);
     }
 
     if (n_args == 2) {
@@ -291,7 +317,7 @@ bi_substr(CELL * sp)
     }
 
     free_STRING(sval);
-    return sp;
+    return_CELL("bi_substr", sp);
 }
 
 /*
@@ -305,6 +331,8 @@ bi_match(CELL * sp)
     char *p;
     size_t length;
 
+    TRACE_FUNC("bi_match", sp);
+
     if (sp->type != C_RE)
 	cast_to_RE(sp);
     if ((--sp)->type < C_STRING)
@@ -330,7 +358,7 @@ bi_match(CELL * sp)
 
     RSTART->dval = sp->dval;
 
-    return sp;
+    return_CELL("bi_match", sp);
 }
 
 CELL *
@@ -339,6 +367,8 @@ bi_toupper(CELL * sp)
     STRING *old;
     register char *p, *q;
 
+    TRACE_FUNC("bi_toupper", sp);
+
     if (sp->type != C_STRING)
 	cast1_to_s(sp);
     old = string(sp);
@@ -352,7 +382,7 @@ bi_toupper(CELL * sp)
 	q++;
     }
     free_STRING(old);
-    return sp;
+    return_CELL("bi_toupper", sp);
 }
 
 CELL *
@@ -361,6 +391,8 @@ bi_tolower(CELL * sp)
     STRING *old;
     register char *p, *q;
 
+    TRACE_FUNC("bi_tolower", sp);
+
     if (sp->type != C_STRING)
 	cast1_to_s(sp);
     old = string(sp);
@@ -374,11 +406,162 @@ bi_tolower(CELL * sp)
 	q++;
     }
     free_STRING(old);
-    return sp;
+    return_CELL("bi_tolower", sp);
 }
 
+/*
+ * Like gawk...
+ */
+CELL *
+bi_systime(CELL * sp)
+{
+    time_t result;
+    time(&result);
+
+    TRACE_FUNC("bi_systime", sp);
+
+    sp++;
+    sp->type = C_DOUBLE;
+    sp->dval = (double) result;
+    return_CELL("bi_systime", sp);
+}
+
+#ifdef HAVE_MKTIME
+/*  mktime(datespec)
+    Turns datespec into a time stamp of the same form as returned by systime(). 
+    The datespec is a string of the form
+        YYYY MM DD HH MM SS [DST].
+*/
+CELL *
+bi_mktime(CELL * sp)
+{
+    time_t result;
+    struct tm my_tm;
+    STRING *sval = string(sp);
+    int error = 0;
+
+    TRACE_FUNC("bi_mktime", sp);
+
+    memset(&my_tm, 0, sizeof(my_tm));
+    switch (sscanf(sval->str, "%d %d %d %d %d %d %d",
+		   &my_tm.tm_year,
+		   &my_tm.tm_mon,
+		   &my_tm.tm_mday,
+		   &my_tm.tm_hour,
+		   &my_tm.tm_min,
+		   &my_tm.tm_sec,
+		   &my_tm.tm_isdst)) {
+    case 7:
+	break;
+    case 6:
+	my_tm.tm_isdst = -1;	/* ask mktime to get timezone */
+	break;
+    default:
+	error = 1;		/* not enough data */
+	break;
+    }
+
+    if (error) {
+	result = -1;
+    } else {
+	my_tm.tm_year -= 1900;
+	my_tm.tm_mon -= 1;
+	result = mktime(&my_tm);
+    }
+    TRACE(("...bi_mktime(%s) ->%s", sval->str, ctime(&result)));
+
+    cell_destroy(sp);
+    sp->type = C_DOUBLE;
+    sp->dval = (double) result;
+    return_CELL("bi_mktime", sp);
+}
+#endif
+
+/*  strftime(format, timestamp, utc) 
+    should be equal to gawk strftime. all parameters are optional:
+        format: ansi c strftime format descriptor. default is "%c"
+        timestamp: seconds since unix epoch. default is now
+        utc: when set and != 0 date is utc otherwise local. default is 0
+*/
+#ifdef HAVE_STRFTIME
+CELL *
+bi_strftime(CELL * sp)
+{
+    const char *format = "%c";
+    time_t rawtime;
+    struct tm *ptm;
+    int n_args;
+    int utc;
+    STRING *sval = 0;		/* strftime(sval->str, timestamp, utc) */
+    char buff[128];
+    size_t result;
+
+    TRACE_FUNC("bi_strftime", sp);
+
+    n_args = sp->type;
+    sp -= n_args;
+
+    if (n_args > 0) {
+	if (sp->type != C_STRING)
+	    cast1_to_s(sp);
+	/* don't use < C_STRING shortcut */
+	sval = string(sp);
+
+	if ((int) sval->len != 0)	/* strftime on valid format */
+	    format = sval->str;
+    } else {
+	sp->type = C_STRING;
+    }
+
+    if (n_args > 1) {
+	if (sp[1].type != C_DOUBLE)
+	    cast1_to_d(sp + 1);
+	rawtime = d_to_i(sp[1].dval);
+    } else {
+	time(&rawtime);
+    }
+
+    if (n_args > 2) {
+	if (sp[2].type != C_DOUBLE)
+	    cast1_to_d(sp + 2);
+	utc = d_to_i(sp[2].dval);
+    } else {
+	utc = 0;
+    }
+
+    if (utc != 0)
+	ptm = gmtime(&rawtime);
+    else
+	ptm = localtime(&rawtime);
+
+    result = strftime(buff, sizeof(buff) / sizeof(buff[0]), format, ptm);
+    TRACE(("...bi_strftime (%s, \"%d.%d.%d %d.%d.%d %d\", %d) ->%s\n",
+	   format,
+	   ptm->tm_year,
+	   ptm->tm_mon,
+	   ptm->tm_mday,
+	   ptm->tm_hour,
+	   ptm->tm_min,
+	   ptm->tm_sec,
+	   ptm->tm_isdst,
+	   utc,
+	   buff));
+
+    if (sval)
+	free_STRING(sval);
+
+    sp->ptr = (PTR) new_STRING1(buff, result);
+
+    while (n_args > 1) {
+	n_args--;
+	cell_destroy(sp + n_args);
+    }
+    return_CELL("bi_strftime", sp);
+}
+#endif /* HAVE_STRFTIME */
+
 /************************************************
-  arithemetic builtins
+  arithmetic builtins
  ************************************************/
 
 #if STDC_MATHERR
@@ -395,145 +578,167 @@ fplib_err(
 CELL *
 bi_sin(CELL * sp)
 {
+    TRACE_FUNC("bi_sin", sp);
+
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
     sp->dval = sin(sp->dval);
 #else
-    double x;
+    {
+	double x;
 
-    errno = 0;
-    if (sp->type != C_DOUBLE)
-	cast1_to_d(sp);
-    x = sp->dval;
-    sp->dval = sin(sp->dval);
-    if (errno)
-	fplib_err("sin", x, "loss of precision");
+	errno = 0;
+	if (sp->type != C_DOUBLE)
+	    cast1_to_d(sp);
+	x = sp->dval;
+	sp->dval = sin(sp->dval);
+	if (errno)
+	    fplib_err("sin", x, "loss of precision");
+    }
 #endif
-    return sp;
+    return_CELL("bi_sin", sp);
 }
 
 CELL *
 bi_cos(CELL * sp)
 {
+    TRACE_FUNC("bi_cos", sp);
+
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
     sp->dval = cos(sp->dval);
 #else
-    double x;
+    {
+	double x;
 
-    errno = 0;
-    if (sp->type != C_DOUBLE)
-	cast1_to_d(sp);
-    x = sp->dval;
-    sp->dval = cos(sp->dval);
-    if (errno)
-	fplib_err("cos", x, "loss of precision");
+	errno = 0;
+	if (sp->type != C_DOUBLE)
+	    cast1_to_d(sp);
+	x = sp->dval;
+	sp->dval = cos(sp->dval);
+	if (errno)
+	    fplib_err("cos", x, "loss of precision");
+    }
 #endif
-    return sp;
+    return_CELL("bi_cos", sp);
 }
 
 CELL *
 bi_atan2(CELL * sp)
 {
+    TRACE_FUNC("bi_atan2", sp);
+
 #if  !	STDC_MATHERR
     sp--;
     if (TEST2(sp) != TWO_DOUBLES)
 	cast2_to_d(sp);
     sp->dval = atan2(sp->dval, (sp + 1)->dval);
 #else
-
-    errno = 0;
-    sp--;
-    if (TEST2(sp) != TWO_DOUBLES)
-	cast2_to_d(sp);
-    sp->dval = atan2(sp->dval, (sp + 1)->dval);
-    if (errno)
-	rt_error("atan2(0,0) : domain error");
+    {
+	errno = 0;
+	sp--;
+	if (TEST2(sp) != TWO_DOUBLES)
+	    cast2_to_d(sp);
+	sp->dval = atan2(sp->dval, (sp + 1)->dval);
+	if (errno)
+	    rt_error("atan2(0,0) : domain error");
+    }
 #endif
-    return sp;
+    return_CELL("bi_atan2", sp);
 }
 
 CELL *
 bi_log(CELL * sp)
 {
+    TRACE_FUNC("bi_log", sp);
+
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
     sp->dval = log(sp->dval);
 #else
-    double x;
+    {
+	double x;
 
-    errno = 0;
-    if (sp->type != C_DOUBLE)
-	cast1_to_d(sp);
-    x = sp->dval;
-    sp->dval = log(sp->dval);
-    if (errno)
-	fplib_err("log", x, "domain error");
+	errno = 0;
+	if (sp->type != C_DOUBLE)
+	    cast1_to_d(sp);
+	x = sp->dval;
+	sp->dval = log(sp->dval);
+	if (errno)
+	    fplib_err("log", x, "domain error");
+    }
 #endif
-    return sp;
+    return_CELL("bi_log", sp);
 }
 
 CELL *
 bi_exp(CELL * sp)
 {
+    TRACE_FUNC("bi_exp", sp);
+
 #if  ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
     sp->dval = exp(sp->dval);
 #else
-    double x;
+    {
+	double x;
 
-    errno = 0;
-    if (sp->type != C_DOUBLE)
-	cast1_to_d(sp);
-    x = sp->dval;
-    sp->dval = exp(sp->dval);
-    if (errno && sp->dval)
-	fplib_err("exp", x, "overflow");
-    /* on underflow sp->dval==0, ignore */
+	errno = 0;
+	if (sp->type != C_DOUBLE)
+	    cast1_to_d(sp);
+	x = sp->dval;
+	sp->dval = exp(sp->dval);
+	if (errno && sp->dval)
+	    fplib_err("exp", x, "overflow");
+	/* on underflow sp->dval==0, ignore */
+    }
 #endif
-    return sp;
+    return_CELL("bi_exp", sp);
 }
 
 CELL *
 bi_int(CELL * sp)
 {
+    TRACE_FUNC("bi_int", sp);
+
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
     sp->dval = sp->dval >= 0.0 ? floor(sp->dval) : ceil(sp->dval);
-    return sp;
+    return_CELL("bi_int", sp);
 }
 
 CELL *
 bi_sqrt(CELL * sp)
 {
+    TRACE_FUNC("bi_sqrt", sp);
+
 #if  ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
     sp->dval = sqrt(sp->dval);
 #else
-    double x;
+    {
+	double x;
 
-    errno = 0;
-    if (sp->type != C_DOUBLE)
-	cast1_to_d(sp);
-    x = sp->dval;
-    sp->dval = sqrt(sp->dval);
-    if (errno)
-	fplib_err("sqrt", x, "domain error");
+	errno = 0;
+	if (sp->type != C_DOUBLE)
+	    cast1_to_d(sp);
+	x = sp->dval;
+	sp->dval = sqrt(sp->dval);
+	if (errno)
+	    fplib_err("sqrt", x, "domain error");
+    }
 #endif
-    return sp;
+    return_CELL("bi_sqrt", sp);
 }
 
-#ifndef NO_TIME_H
-#include <time.h>
+#if defined(mawk_srand) || defined(mawk_rand)
+#define USE_SYSTEM_SRAND
 #else
-#include <sys/types.h>
-#endif
-
 /* For portability, we'll use our own random number generator , taken
    from:  Park, SK and Miller KW, "Random Number Generators:
    Good Ones are Hard to Find", CACM, 31, 1192-1201, 1988.
@@ -558,14 +763,25 @@ static CELL cseed;		/* argument of last call to srand() */
 			  if ( t >= M ) t = (t+M)&M ;\
 			  s = (long) t ;\
 			}
-#endif
+#endif /* M == MAX__LONG */
+#endif /* defined(mawk_srand) || defined(mawk_rand) */
 
 CELL *
 bi_srand(CELL * sp)
 {
+#ifdef USE_SYSTEM_SRAND
+    static long seed = 1;
+    static CELL cseed =
+    {
+	C_DOUBLE, 0, 0, 1.0
+    };
+#endif
+
     CELL c;
 
-    if (sp->type == 0)		/* seed off clock */
+    TRACE_FUNC("bi_srand", sp);
+
+    if (sp->type == C_NOINIT)	/* seed off clock */
     {
 	time_t secs = time((time_t *) 0);
 	cellcpy(sp, &cseed);
@@ -580,6 +796,10 @@ bi_srand(CELL * sp)
 	cseed = c;
     }
 
+#ifdef USE_SYSTEM_SRAND
+    seed = d_to_i(cseed.dval);
+    mawk_srand((unsigned) seed);
+#else
     /* The old seed is now in *sp ; move the value in cseed to
        seed in range [1,M) */
 
@@ -598,19 +818,33 @@ bi_srand(CELL * sp)
     /* crank it once so close seeds don't give a close
        first result  */
     crank(seed);
+#endif
 
-    return sp;
+    return_CELL("bi_srand", sp);
 }
 
 CELL *
 bi_rand(CELL * sp)
 {
+    TRACE_FUNC("bi_rand", sp);
+
+#ifdef USE_SYSTEM_SRAND
+    {
+	long value = (long) mawk_rand();
+	sp++;
+	sp->type = C_DOUBLE;
+	sp->dval = ((double) value) / RAND_MAX;
+    }
+#else
     crank(seed);
     sp++;
     sp->type = C_DOUBLE;
     sp->dval = (double) seed / (double) M;
-    return sp;
+#endif
+
+    return_CELL("bi_rand", sp);
 }
+
 #undef	 A
 #undef	 M
 #undef   MX
@@ -629,13 +863,16 @@ bi_close(CELL * sp)
 {
     int x;
 
+    TRACE_FUNC("bi_close", sp);
+
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
     x = file_close((STRING *) sp->ptr);
     free_STRING(string(sp));
     sp->type = C_DOUBLE;
     sp->dval = (double) x;
-    return sp;
+
+    return_CELL("bi_close", sp);
 }
 
 CELL *
@@ -643,6 +880,8 @@ bi_fflush(CELL * sp)
 {
     int ret = 0;
 
+    TRACE_FUNC("bi_fflush", sp);
+
     if (sp->type == 0)
 	fflush(stdout);
     else {
@@ -655,7 +894,8 @@ bi_fflush(CELL * sp)
 
     sp->type = C_DOUBLE;
     sp->dval = (double) ret;
-    return sp;
+
+    return_CELL("bi_fflush", sp);
 }
 
 CELL *
@@ -665,6 +905,8 @@ bi_system(CELL * sp GCC_UNUSED)
     int pid;
     unsigned ret_val;
 
+    TRACE_FUNC("bi_system", sp);
+
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
 
@@ -691,7 +933,6 @@ bi_system(CELL * sp GCC_UNUSED)
     cell_destroy(sp);
     sp->type = C_DOUBLE;
     sp->dval = (double) ret_val;
-    return sp;
 #elif defined(MSDOS)
     int retval;
 
@@ -701,10 +942,10 @@ bi_system(CELL * sp GCC_UNUSED)
     free_STRING(string(sp));
     sp->type = C_DOUBLE;
     sp->dval = (double) retval;
-    return sp;
 #else
-    return 0;
+    sp = 0;
 #endif
+    return_CELL("bi_system", sp);
 }
 
 /*  getline()  */
@@ -724,6 +965,8 @@ bi_getline(CELL * sp)
     size_t len;
     FIN *fin_p;
 
+    TRACE_FUNC("bi_getline", sp);
+
     switch (sp->type) {
     case 0:
 	sp--;
@@ -808,8 +1051,10 @@ bi_getline(CELL * sp)
   eof:
     sp->dval = 0.0;		/* fall thru to done  */
 
-  done:sp->type = C_DOUBLE;
-    return sp;
+  done:
+    sp->type = C_DOUBLE;
+
+    return_CELL("bi_getline", sp);
 }
 
 /**********************************************
@@ -830,6 +1075,8 @@ bi_sub(CELL * sp)
     char *front, *middle, *back;	/* pieces */
     size_t front_len, middle_len, back_len;
 
+    TRACE_FUNC("bi_sub", sp);
+
     sp -= 2;
     if (sp->type != C_RE)
 	cast_to_RE(sp);
@@ -883,7 +1130,8 @@ bi_sub(CELL * sp)
     repl_destroy(sp + 1);
     sp->type = C_DOUBLE;
     sp->dval = middle != (char *) 0 ? 1.0 : 0.0;
-    return sp;
+
+    return_CELL("bi_sub", sp);
 }
 
 typedef enum {
@@ -991,7 +1239,7 @@ old_gsub(PTR re, int level)
 		xbuff[0] = *ThisTarget;
 		xbuff[1] = 0;
 		ThisReplace.type = C_REPL;
-		ThisReplace.ptr = (PTR) new_STRING1(xbuff, 1);
+		ThisReplace.ptr = (PTR) new_STRING1(xbuff, (size_t) 1);
 
 		NextTarget = ThisTarget + 1;
 		NextTargetLen = ThisTargetLen - 1;
@@ -1093,7 +1341,7 @@ new_gsub(PTR re, int level)
 		xbuff[0] = *ThisTarget;
 		xbuff[1] = 0;
 		ThisReplace.type = C_REPL;
-		ThisReplace.ptr = (PTR) new_STRING1(xbuff, 1);
+		ThisReplace.ptr = (PTR) new_STRING1(xbuff, (size_t) 1);
 
 		NextTarget = ThisTarget + 1;
 		NextTargetLen = ThisTargetLen - 1;
@@ -1196,6 +1444,8 @@ bi_gsub(CELL * sp)
     size_t stack_needs;
     int level = 0;
 
+    TRACE_FUNC("bi_gsub", sp);
+
     sp -= 2;
     if (sp->type != C_RE)
 	cast_to_RE(sp);
@@ -1270,5 +1520,6 @@ bi_gsub(CELL * sp)
 
     sp->type = C_DOUBLE;
     sp->dval = (double) repl_cnt;
-    return sp;
+
+    return_CELL("bi_gsub", sp);
 }
