@@ -1,6 +1,6 @@
 /********************************************
 error.c
-copyright 2008-2009,2010 Thomas E. Dickey
+copyright 2008-2010,2012 Thomas E. Dickey
 copyright 1991-1994,1995 Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: error.c,v 1.15 2010/12/10 17:00:00 tom Exp $
+ * $MawkId: error.c,v 1.20 2012/11/28 09:55:29 tom Exp $
  * @Log: error.c,v @
  * Revision 1.6  1995/06/06  00:18:22  mike
  * change mawk_exit(1) to mawk_exit(2)
@@ -42,22 +42,12 @@ the GNU General Public License, version 2, 1991.
  *
 */
 
-#include  "mawk.h"
-#include  "scan.h"
-#include  "bi_vars.h"
+#include <mawk.h>
+#include <scan.h>
+#include <bi_vars.h>
 
 #include <stdarg.h>
 
-#ifndef  EOF
-#define  EOF  (-1)
-#endif
-
-static void rt_where(void);
-
-#ifdef  NO_VFPRINTF
-#define  vfprintf  simple_vfprintf
-#endif
-
 /* for run time error messages only */
 unsigned rt_nr, rt_fnr;
 /* *INDENT-OFF* */
@@ -118,11 +108,15 @@ static struct token_str {
 
 /* if paren_cnt >0 and we see one of these, we are missing a ')' */
 static int missing_rparen[] =
-{EOF, NL, SEMI_COLON, SC_FAKE_SEMI_COLON, RBRACE, 0};
+{
+    EOF, NL, SEMI_COLON, SC_FAKE_SEMI_COLON, RBRACE, 0
+};
 
 /* ditto for '}' */
 static int missing_rbrace[] =
-{EOF, BEGIN, END, 0};
+{
+    EOF, BEGIN, END, 0
+};
 
 static void
 missing(int c, const char *n, unsigned ln)
@@ -242,6 +236,9 @@ compile_error(const char *format,...)
 	s0 = s1 = "";
     }
 
+#ifdef DEBUG
+    fflush(stdout);
+#endif
     fprintf(stderr, "%s: %s%sline %u: ", progname, s0, s1, token_lineno);
     va_start(args, format);
     vfprintf(stderr, format, args);
@@ -251,20 +248,6 @@ compile_error(const char *format,...)
 	mawk_exit(2);
 }
 
-void
-rt_error(const char *format,...)
-{
-    va_list args;
-
-    fprintf(stderr, "%s: run time error: ", progname);
-    va_start(args, format);
-    vfprintf(stderr, format, args);
-    va_end(args);
-    putc('\n', stderr);
-    rt_where();
-    mawk_exit(2);
-}
-
 void
 bozo(const char *s)
 {
@@ -291,6 +274,20 @@ rt_where(void)
 	    string(FILENAME)->str, rt_fnr, rt_nr);
 }
 
+void
+rt_error(const char *format,...)
+{
+    va_list args;
+
+    fprintf(stderr, "%s: run time error: ", progname);
+    va_start(args, format);
+    vfprintf(stderr, format, args);
+    va_end(args);
+    putc('\n', stderr);
+    rt_where();
+    mawk_exit(2);
+}
+
 /* run time */
 void
 rt_overflow(const char *s, unsigned size)
@@ -312,24 +309,48 @@ unexpected_char(void)
 	fprintf(stderr, "unexpected character 0x%02x\n", c);
 }
 
-static const char *
+const char *
 type_to_str(int type)
 {
     const char *retval = 0;
 
     switch (type) {
+    case ST_NONE:
+	retval = "untyped variable";
+	break;
     case ST_VAR:
 	retval = "variable";
 	break;
+    case ST_KEYWORD:
+	retval = "keyword";
+	break;
+    case ST_BUILTIN:
+	retval = "builtin";
+	break;
     case ST_ARRAY:
 	retval = "array";
 	break;
+    case ST_FIELD:
+	retval = "field";
+	break;
+    case ST_NR:
+	retval = "NR";
+	break;
+    case ST_ENV:
+	retval = "ENVIRON";
+	break;
     case ST_FUNCT:
 	retval = "function";
 	break;
+    case ST_LENGTH:
+	retval = "length";
+	break;
     case ST_LOCAL_VAR:
 	retval = "local variable";
 	break;
+    case ST_LOCAL_NONE:
+	retval = "local untyped variable";
+	break;
     case ST_LOCAL_ARRAY:
 	retval = "local array";
 	break;
@@ -346,109 +367,3 @@ type_error(SYMTAB * p)
     compile_error("illegal reference to %s %s",
 		  type_to_str(p->type), p->name);
 }
-
-#ifdef  NO_VFPRINTF
-
-/* a minimal vfprintf  */
-int
-simple_vfprintf(FILE *fp, char *format, va_list argp)
-{
-    char *q, *p, *t;
-    int l_flag;
-    char xbuff[64];
-
-    q = format;
-    xbuff[0] = '%';
-
-    while (*q != 0) {
-	if (*q != '%') {
-	    putc(*q, fp);
-	    q++;
-	    continue;
-	}
-
-	/* mark the start with p */
-	p = ++q;
-	t = xbuff + 1;
-
-	if (*q == '-')
-	    *t++ = *q++;
-	while (scan_code[*(unsigned char *) q] == SC_DIGIT)
-	    *t++ = *q++;
-	if (*q == '.') {
-	    *t++ = *q++;
-	    while (scan_code[*(unsigned char *) q] == SC_DIGIT)
-		*t++ = *q++;
-	}
-
-	if (*q == 'l') {
-	    l_flag = 1;
-	    *t++ = *q++;
-	} else
-	    l_flag = 0;
-
-	*t = *q++;
-	t[1] = 0;
-
-	switch (*t) {
-	case 'c':
-	case 'd':
-	case 'o':
-	case 'x':
-	case 'u':
-	    if (l_flag)
-		fprintf(fp, xbuff, va_arg(argp, long));
-	    else
-		fprintf(fp, xbuff, va_arg(argp, int));
-	    break;
-
-	case 's':
-	    fprintf(fp, xbuff, va_arg(argp, char *));
-	    break;
-
-	case 'g':
-	case 'f':
-	    fprintf(fp, xbuff, va_arg(argp, double));
-	    break;
-
-	default:
-	    putc('%', fp);
-	    q = p;
-	    break;
-	}
-    }
-    return 0;			/* shut up */
-}
-
-#endif /* USE_SIMPLE_VFPRINTF */
-
-#if OPT_TRACE > 0
-static FILE *trace_fp;
-
-void
-Trace(const char *format,...)
-{
-    va_list args;
-
-    if (trace_fp == 0)
-	trace_fp = fopen("Trace.out", "w");
-
-    if (trace_fp == 0)
-	rt_error("cannot open Trace.out");
-
-    va_start(args, format);
-    vfprintf(trace_fp, format, args);
-    va_end(args);
-}
-
-#ifdef NO_LEAKS
-void
-trace_leaks(void)
-{
-    if (trace_fp != 0) {
-	fclose(trace_fp);
-	trace_fp = 0;
-    }
-}
-#endif
-#endif
