@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: execute.c,v 1.26 2012/06/27 17:15:52 tom Exp $
+ * $MawkId: execute.c,v 1.29 2012/11/05 10:53:43 tom Exp $
  * @Log: execute.c,v @
  * Revision 1.13  1996/02/01  04:39:40  mike
  * dynamic array scheme
@@ -102,6 +102,7 @@ the GNU General Public License, version 2, 1991.
 #include "regexp.h"
 #include "repl.h"
 #include "fin.h"
+
 #include <math.h>
 
 static int compare(CELL *);
@@ -1087,6 +1088,12 @@ execute(INST * cdp,		/* code ptr, start execution here */
 	    sp->dval = t ? 1.0 : 0.0;
 	    break;
 
+	case A_LENGTH:
+	    sp--;
+	    sp->type = C_DOUBLE;
+	    sp->dval = (double) (((ARRAY) ((sp + 0)->ptr))->size);
+	    break;
+
 	case A_TEST:
 	    /* entry :  sp[0].ptr-> an array
 	       sp[-1]  is an expression
