@@ -1,6 +1,7 @@
 /********************************************
 field.c
-copyright 1991, Michael D. Brennan
+copyright 2008-2010, Thomas E. Dickey
+copyright 1991-1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: field.c,v 1.10 2009/12/17 00:20:11 tom Exp $
+ * $MawkId: field.c,v 1.25 2010/12/10 17:00:00 tom Exp $
  * @Log: field.c,v @
  * Revision 1.5  1995/06/18  19:17:47  mike
  * Create a type Int which on most machines is an int, but on machines
@@ -202,7 +203,7 @@ field_init(void)
 }
 
 void
-set_field0(char *s, unsigned len)
+set_field0(char *s, size_t len)
 {
     cell_destroy(&field[0]);
     nf = -1;
@@ -238,15 +239,15 @@ split_field0(void)
     else {
 	switch (fs_shadow.type) {
 	case C_SNULL:		/* FS == "" */
-	    nf = null_split(string(cp)->str);
+	    nf = (int) null_split(string(cp)->str, string(cp)->len);
 	    break;
 
 	case C_SPACE:
-	    nf = space_split(string(cp)->str, string(cp)->len);
+	    nf = (int) space_split(string(cp)->str, string(cp)->len);
 	    break;
 
 	default:
-	    nf = re_split(string(cp), fs_shadow.ptr);
+	    nf = (int) re_split(string(cp), fs_shadow.ptr);
 	    break;
 	}
 
@@ -265,6 +266,7 @@ split_field0(void)
     while (cnt > 0) {
 	cell_destroy(field + cnt);
 	field[cnt].ptr = (PTR) split_buff[cnt - 1];
+	USED_SPLIT_BUFF(cnt - 1);
 	field[cnt--].type = C_MBSTRN;
     }
 
@@ -304,7 +306,7 @@ field_assign(CELL * fp, CELL * cp)
     }
 #endif
 
-    switch (i = (fp - field)) {
+    switch (i = (int) (fp - field)) {
 
     case NF_field:
 
@@ -421,7 +423,7 @@ build_field0(void)
     } else {
 	CELL c;
 	STRING *ofs, *tail;
-	unsigned len;
+	size_t len;
 	register CELL *cp;
 	register char *p, *q;
 	int cnt;
@@ -433,7 +435,7 @@ build_field0(void)
 	tail = (STRING *) c.ptr;
 	cnt = nf - 1;
 
-	len = ((unsigned) cnt) * ofs->len + tail->len;
+	len = ((size_t) cnt) * ofs->len + tail->len;
 
 	fbp = fbank;
 	cp_limit = field + FBANK_SZ;
@@ -549,7 +551,7 @@ field_addr_to_index(CELL * cp)
 	      cp < *p || cp >= *p + FBANK_SZ)
 	p++;
 
-    return ((p - fbank) << FB_SHIFT) + (cp - *p);
+    return (int) (((p - fbank) << FB_SHIFT) + (cp - *p));
 }
 
 /*------- more than 1 fbank needed  ------------*/
@@ -662,3 +664,43 @@ set_binmode(int x)
 }
 
 #endif /* USE_BINMODE */
+
+#ifdef NO_LEAKS
+void
+field_leaks(void)
+{
+    int n;
+
+    free_STRING(string(CONVFMT));
+    free_STRING(string(FS));
+    free_STRING(string(OFMT));
+    free_STRING(string(RS));
+    cell_destroy(&field[0]);
+
+    for (n = 1; n <= nf; ++n) {
+	cell_destroy(&field[n]);
+    }
+
+    switch (fs_shadow.type) {
+    case C_RE:
+	re_destroy(fs_shadow.ptr);
+	break;
+    case C_STRING:
+    case C_STRNUM:
+    case C_MBSTRN:
+	cell_destroy(&fs_shadow);
+	break;
+    default:
+	break;
+    }
+
+    switch (rs_shadow.type) {
+    case SEP_STR:
+	free_STRING(((STRING *) (&rs_shadow.ptr)));
+	break;
+    case SEP_RE:
+	re_destroy(rs_shadow.ptr);
+	break;
+    }
+}
+#endif
