@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: main.c,v 1.23 2010/12/10 17:00:00 tom Exp $
+ * $MawkId: main.c,v 1.25 2012/11/02 23:23:34 tom Exp $
  * @Log: main.c,v @
  * Revision 1.4  1995/06/09  22:57:19  mike
  * parse() no longer returns on error
@@ -57,20 +57,17 @@ the GNU General Public License, version 2, 1991.
 short mawk_state;		/* 0 is compiling */
 int exit_code;
 
-static void
-initialize_locale(void)
+int
+main(int argc, char **argv)
 {
 #ifdef LOCALE
     setlocale(LC_CTYPE, "");
     setlocale(LC_NUMERIC, "C");
 #endif
-}
-
-int
-main(int argc, char **argv)
-{
-    initialize_locale();
     initialize(argc, argv);
+#ifdef LOCALE
+    setlocale(LC_NUMERIC, "");
+#endif
 
     parse();
 
