@@ -1,6 +1,7 @@
 /********************************************
-dosexec.h
-copyright 1991, Michael D. Brennan
+dosexec.c
+copyright 2009,2010, Thomas E. Dickey
+copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,8 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: dosexec.c,v 1.2 2009/07/23 23:04:08 tom Exp $
+ * $MawkId: dosexec.c,v 1.4 2010/12/10 17:00:00 tom Exp $
+ *
  * @Log: dosexec.c,v @
  * Revision 1.3  1995/08/20  16:37:22  mike
  * exit(1) -> exit(2)
@@ -34,21 +36,20 @@ the GNU General Public License, version 2, 1991.
  *
  * Revision 1.1  91/10/29  09:45:56  brennan
  * Initial revision
- * 
  */
 
-/* system() and pipes() for MSDOS */
+/* system() and pipes() for MSDOS and Win32 console */
 
 #include "mawk.h"
 
-#if MSDOS
+#if defined(MSDOS) || defined(_WINNT)
 #include "memory.h"
 #include "files.h"
 #include "fin.h"
 
 #include <process.h>
 
-static char *shell;		/* e.g.   "c:\\sys\\command.com"  */
+static char *my_shell;		/* e.g.   "c:\\sys\\command.com"  */
 static char *command_opt;	/*  " /c"  */
 
 static void
@@ -63,12 +64,12 @@ get_shell(void)
 	while (*p != ' ' && *p != '\t')
 	    p++;
 	len = p - s;
-	shell = (char *) zmalloc(len + 1);
-	memcpy(shell, s, len);
-	shell[len] = 0;
+	my_shell = (char *) zmalloc(len + 1);
+	memcpy(my_shell, s, len);
+	my_shell[len] = 0;
 	command_opt = p;
     } else if ((s = getenv("COMSPEC")) != 0) {
-	shell = s;
+	my_shell = s;
 	command_opt = " /c";
 	/* leading space needed because of bug in command.com */
     } else {
@@ -83,7 +84,7 @@ DOSexec(char *command)
 {
     char xbuff[256];
 
-    if (!shell)
+    if (!my_shell)
 	get_shell();
 
     sprintf(xbuff, "%s %s", command_opt, command);
@@ -91,7 +92,7 @@ DOSexec(char *command)
     fflush(stderr);
     fflush(stdout);
 
-    return spawnl(P_WAIT, shell, shell, xbuff, (char *) 0);
+    return spawnl(P_WAIT, my_shell, my_shell, xbuff, (char *) 0);
 }
 
 static int next_tmp;		/* index for naming temp files */
@@ -106,11 +107,12 @@ tmp_file_name(int id,
     if (mawkid == 0) {
 	/* first time */
 	union {
-	    void far *ptr;
+	    void *ptr;
 	    unsigned w[2];
 	} xptr;
 
-	xptr.ptr = (void far *) &mawkid;
+	memset(&xptr, 0, sizeof(xptr));
+	xptr.ptr = (void *) &mawkid;
 	mawkid = xptr.w[1];
 
 	tmpdir = getenv("MAWKTMPDIR");
@@ -149,7 +151,7 @@ get_pipe(char *command,
     return retval;
 }
 
-/* closing a fake pipes involves running the out pipe 
+/* closing a fake pipes involves running the out pipe
    command
 */
 
