@@ -1,6 +1,7 @@
 /********************************************
 split.c
-copyright 1991, Michael D. Brennan
+copyright 2008-2009,2010, Thomas E. Dickey
+copyright 1991-1993,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: split.c,v 1.14 2009/12/15 10:59:30 tom Exp $
+ * $MawkId: split.c,v 1.21 2010/12/10 17:00:00 tom Exp $
  * @Log: split.c,v @
  * Revision 1.3  1996/02/01  04:39:42  mike
  * dynamic array scheme
@@ -67,24 +68,6 @@ the GNU General Public License, version 2, 1991.
 
 SPLIT_OV *split_ov_list;
 
-static int space_ov_split(char *, char *);
-static int null_ov_split(char *);
-
-/*
- * Split string s of length slen on SPACE without changing s.
- * Load the pieces into STRINGS and ptrs into split_buff[].
- *
- * return the number of pieces
- */
-int
-space_split(char *s, unsigned slen)
-{
-    char *back = s + slen;
-    int i = 0;
-    char *q;
-    STRING *sval;
-    int lcnt = MAX_SPLIT / 3;
-
 #define EAT_SPACE()   while ( scan_code[*(unsigned char*)s] ==\
 			      SC_SPACE )  s++
 #define EAT_NON_SPACE()	  \
@@ -92,45 +75,14 @@ space_split(char *s, unsigned slen)
     while ( scan_code[*(unsigned char*)s] != SC_SPACE )	 s++ ;\
     *back = 0
 
-    while (lcnt--) {
-	EAT_SPACE();
-	if (*s == 0)
-	    goto done;
-	/* mark the front with q */
-	q = s++;
-	EAT_NON_SPACE();
-	sval = split_buff[i++] = new_STRING1(q, (unsigned) (s - q));
-
-	EAT_SPACE();
-	if (*s == 0)
-	    goto done;
-	q = s++;
-	EAT_NON_SPACE();
-	sval = split_buff[i++] = new_STRING1(q, (unsigned) (s - q));
-
-	EAT_SPACE();
-	if (*s == 0)
-	    goto done;
-	q = s++;
-	EAT_NON_SPACE();
-	sval = split_buff[i++] = new_STRING1(q, (unsigned) (s - q));
-
-    }
-    /* we've overflowed */
-    return i + space_ov_split(s, back);
-
-  done:
-    return i;
-}
-
-static int
+static size_t
 space_ov_split(char *s, char *back)
 {
     SPLIT_OV dummy;
     register SPLIT_OV *tail = &dummy;
     char *q;
-    int cnt = 0;
-    unsigned len;
+    size_t cnt = 0;
+    size_t len;
 
     while (1) {
 	EAT_SPACE();
@@ -140,7 +92,7 @@ space_ov_split(char *s, char *back)
 	EAT_NON_SPACE();
 
 	tail = tail->link = ZMALLOC(SPLIT_OV);
-	tail->sval = new_STRING0(len = (unsigned) (s - q));
+	tail->sval = new_STRING0(len = (size_t) (s - q));
 	memcpy(tail->sval->str, q, len);
 	cnt++;
     }
@@ -150,11 +102,56 @@ space_ov_split(char *s, char *back)
     return cnt;
 }
 
+/*
+ * Split string s of length slen on SPACE without changing s.
+ * Load the pieces into STRINGS and ptrs into split_buff[].
+ *
+ * return the number of pieces
+ */
+size_t
+space_split(char *s, size_t slen)
+{
+    char *back = s + slen;
+    size_t i = 0;
+    char *q;
+    int lcnt = MAX_SPLIT / 3;
+
+    while (lcnt--) {
+	EAT_SPACE();
+	if (*s == 0)
+	    goto done;
+	/* mark the front with q */
+	q = s++;
+	EAT_NON_SPACE();
+	split_buff[i++] = new_STRING1(q, (size_t) (s - q));
+
+	EAT_SPACE();
+	if (*s == 0)
+	    goto done;
+	q = s++;
+	EAT_NON_SPACE();
+	split_buff[i++] = new_STRING1(q, (size_t) (s - q));
+
+	EAT_SPACE();
+	if (*s == 0)
+	    goto done;
+	q = s++;
+	EAT_NON_SPACE();
+	split_buff[i++] = new_STRING1(q, (size_t) (s - q));
+
+    }
+    /* we've overflowed */
+    return i + space_ov_split(s, back);
+
+  done:
+    return i;
+}
+
 /* match a string with a regular expression, but
  * only matches of positive length count
  */
 char *
-re_pos_match(char *s, size_t str_len, PTR re, unsigned *lenp)
+re_pos_match(char *s, size_t str_len, PTR re, size_t *lenp)
 {
     char *result = 0;
 
@@ -178,26 +175,26 @@ re_pos_match(char *s, size_t str_len, PTR re, unsigned *lenp)
  *
  *  Return number of pieces.
  */
-static int
+static size_t
 re_ov_split(char *s, size_t slen, PTR re)
 {
     SPLIT_OV dummy;
     SPLIT_OV *tail = &dummy;
-    int cnt = 1;
+    size_t cnt = 1;
     char *limit = s + slen;
     char *t;
-    unsigned mlen;
+    size_t mlen;
 
     while ((s < limit)
 	   && (t = re_pos_match(s, (size_t) (limit - s), re, &mlen))) {
 	tail = tail->link = ZMALLOC(SPLIT_OV);
-	tail->sval = new_STRING1(s, (unsigned) (t - s));
+	tail->sval = new_STRING1(s, (size_t) (t - s));
 	s = t + mlen;
 	cnt++;
     }
     /* and one more */
     tail = tail->link = ZMALLOC(SPLIT_OV);
-    tail->sval = new_STRING1(s, (unsigned) (limit - s));
+    tail->sval = new_STRING1(s, (size_t) (limit - s));
     tail->link = (SPLIT_OV *) 0;
     split_ov_list = dummy.link;
 
@@ -207,7 +204,7 @@ re_ov_split(char *s, size_t slen, PTR re)
 #define RE_SPLIT3 \
 	if (!(t = re_pos_match(s, slen, re, &mlen))) \
 	    goto done; \
-	sval = split_buff[i++] = new_STRING1(s, (unsigned) (t - s)); \
+	split_buff[i++] = new_STRING1(s, (size_t) (t - s)); \
 	s = t + mlen; \
 	if (s > limit) { \
 	    slen = (size_t) (-1); \
@@ -215,16 +212,15 @@ re_ov_split(char *s, size_t slen, PTR re)
 	} \
 	slen = (size_t) (limit - s)
 
-int
+size_t
 re_split(STRING * s_param, PTR re)
 {
     char *limit = s_param->str + s_param->len;
     char *s = s_param->str;
     char *t;
-    int i = 0;
+    size_t i = 0;
     size_t slen = s_param->len;
-    unsigned mlen;
-    STRING *sval;
+    size_t mlen;
     int lcnt = MAX_SPLIT / 3;
 
     while (lcnt--) {
@@ -242,43 +238,47 @@ re_split(STRING * s_param, PTR re)
     return i;
 }
 
-int
-null_split(char *s)
-{
-    int cnt = 0;		/* number of fields split */
-    STRING *sval;
-    int i = 0;			/* indexes split_buff[] */
-
-    while (*s) {
-	if (cnt == MAX_SPLIT)
-	    return cnt + null_ov_split(s);
-
-	sval = new_STRING0(1);
-	sval->str[0] = *s++;
-	split_buff[i++] = sval;
-	cnt++;
-    }
-    return cnt;
-}
-
-static int
-null_ov_split(char *s)
+static size_t
+null_ov_split(char *s, size_t slen)
 {
     SPLIT_OV dummy;
     SPLIT_OV *ovp = &dummy;
-    int cnt = 0;
+    size_t cnt = 0;
 
-    while (*s) {
+    while (slen) {
 	ovp = ovp->link = ZMALLOC(SPLIT_OV);
-	ovp->sval = new_STRING0(1);
+	ovp->sval = new_STRING0((size_t) 1);
 	ovp->sval->str[0] = *s++;
 	cnt++;
+	--slen;
     }
     ovp->link = (SPLIT_OV *) 0;
     split_ov_list = dummy.link;
     return cnt;
 }
 
+size_t
+null_split(char *s, size_t slen)
+{
+    size_t cnt = 0;		/* number of fields split */
+    STRING *sval;
+    int i = 0;			/* indexes split_buff[] */
+
+    while (slen) {
+	if (cnt == MAX_SPLIT) {
+	    cnt += null_ov_split(s, slen);
+	    break;
+	} else {
+	    sval = new_STRING0((size_t) 1);
+	    sval->str[0] = *s++;
+	    split_buff[i++] = sval;
+	    cnt++;
+	    --slen;
+	}
+    }
+    return cnt;
+}
+
 /*  split(s, X, r)
  *  split s into array X on r
  *
@@ -289,7 +289,7 @@ null_ov_split(char *s)
 CELL *
 bi_split(CELL * sp)
 {
-    int cnt = 0;		/* the number of pieces */
+    size_t cnt = 0;		/* the number of pieces */
 
     if (sp->type < C_RE)
 	cast_for_split(sp);
@@ -311,7 +311,7 @@ bi_split(CELL * sp)
 	    break;
 
 	case C_SNULL:		/* split on empty string */
-	    cnt = null_split(string(sp)->str);
+	    cnt = null_split(string(sp)->str, string(sp)->len);
 	    break;
 
 	default:
